@@ -395,9 +395,18 @@ def _donormalize(patterns, default, root, cwd, auditor=None, warn=None):
 
 class basematcher:
     def __init__(self, badfn=None):
+        self._was_tampered_with = False
         if badfn is not None:
             self.bad = badfn
 
+    def was_tampered_with_nonrec(self):
+        # [_was_tampered_with] is used to track if when extensions changed the matcher
+        # behavior (crazy stuff!), so we disable the rust fast path.
+        return self._was_tampered_with
+
+    def was_tampered_with(self):
+        return self.was_tampered_with_nonrec()
+
     def __call__(self, fn):
         return self.matchfn(fn)
 
@@ -638,6 +647,11 @@ class patternmatcher(basematcher):
         super(patternmatcher, self).__init__(badfn)
         kindpats.sort()
 
+        if rustmod is not None:
+            # We need to pass the patterns to Rust because they can contain
+            # patterns from the user interface
+            self._kindpats = kindpats
+
         roots, dirs, parents = _rootsdirsandparents(kindpats)
         self._files = _explicitfiles(kindpats)
         self._dirs_explicit = set(dirs)
@@ -880,6 +894,13 @@ class differencematcher(basematcher):
         self.bad = m1.bad
         self.traversedir = m1.traversedir
 
+    def was_tampered_with(self):
+        return (
+            self.was_tampered_with_nonrec()
+            or self._m1.was_tampered_with()
+            or self._m2.was_tampered_with()
+        )
+
     def matchfn(self, f):
         return self._m1(f) and not self._m2(f)
 
@@ -963,6 +984,13 @@ class intersectionmatcher(basematcher):
         self.bad = m1.bad
         self.traversedir = m1.traversedir
 
+    def was_tampered_with(self):
+        return (
+            self.was_tampered_with_nonrec()
+            or self._m1.was_tampered_with()
+            or self._m2.was_tampered_with()
+        )
+
     @propertycache
     def _files(self):
         if self.isexact():
@@ -1060,6 +1088,11 @@ class subdirmatcher(basematcher):
         if matcher.prefix():
             self._always = any(f == path for f in matcher._files)
 
+    def was_tampered_with(self):
+        return (
+            self.was_tampered_with_nonrec() or self._matcher.was_tampered_with()
+        )
+
     def bad(self, f, msg):
         self._matcher.bad(self._path + b"/" + f, msg)
 
@@ -1194,6 +1227,11 @@ class unionmatcher(basematcher):
         self.traversedir = m1.traversedir
         self._matchers = matchers
 
+    def was_tampered_with(self):
+        return self.was_tampered_with_nonrec() or any(
+            map(lambda m: m.was_tampered_with(), self._matchers)
+        )
+
     def matchfn(self, f):
         for match in self._matchers:
             if match(f):
